package org.apache.bcel.util;

import org.apache.bcel.classfile.Utility;
import org.apache.bcel.classfile.ClassParser;
import java.io.InputStream;
import java.io.IOException;
import org.apache.bcel.classfile.JavaClass;

abstract class AbstractClassPathRepository implements Repository
{
    private final ClassPath classPath;
    
    AbstractClassPathRepository(final ClassPath classPath) {
        this.classPath = classPath;
    }
    
    @Override
    public abstract void clear();
    
    @Override
    public abstract JavaClass findClass(final String p0);
    
    @Override
    public ClassPath getClassPath() {
        return this.classPath;
    }
    
    @Override
    public JavaClass loadClass(final Class<?> clazz) throws ClassNotFoundException {
        final String className = clazz.getName();
        final JavaClass repositoryClass = this.findClass(className);
        if (repositoryClass != null) {
            return repositoryClass;
        }
        String name = className;
        final int i = name.lastIndexOf(46);
        if (i > 0) {
            name = name.substring(i + 1);
        }
        try (InputStream clsStream = clazz.getResourceAsStream(name + ".class")) {
            return this.loadClass(clsStream, className);
        }
        catch (final IOException e) {
            return null;
        }
    }
    
    private JavaClass loadClass(final InputStream inputStream, final String className) throws ClassNotFoundException {
        try {
            if (inputStream != null) {
                final ClassParser parser = new ClassParser(inputStream, className);
                final JavaClass clazz = parser.parse();
                this.storeClass(clazz);
                return clazz;
            }
        }
        catch (final IOException e) {
            throw new ClassNotFoundException("Exception while looking for class " + className + ": " + e, e);
        }
        throw new ClassNotFoundException("ClassRepository could not load " + className);
    }
    
    @Override
    public JavaClass loadClass(String className) throws ClassNotFoundException {
        if (className == null || className.isEmpty()) {
            throw new IllegalArgumentException("Invalid class name " + className);
        }
        className = Utility.pathToPackage(className);
        final JavaClass clazz = this.findClass(className);
        if (clazz != null) {
            return clazz;
        }
        try (InputStream inputStream = this.classPath.getInputStream(className)) {
            return this.loadClass(inputStream, className);
        }
        catch (final IOException e) {
            throw new ClassNotFoundException("Exception while looking for class " + className + ": " + e, e);
        }
    }
    
    @Override
    public abstract void removeClass(final JavaClass p0);
    
    @Override
    public abstract void storeClass(final JavaClass p0);
}
